// TI File $Revision: /main/$
// Checkin $Date: June 11, 2012   10:16:38 $
// Author: Dale Li
//#############################################################################		
// FILE:   main.c
// TITLE:  DSP2833x program for ADS833x ADC.  
//    This program requires the DSP2833x header files and is configured for "boot to SARAM" operation.
//
// DESCRIPTION:
//    This example configures CPU Timer0 to create a periodical interupt, and toggles the GPIO12
//    once per interrupt to create /CONVST signal to ADS833x.At the same time,this example monitors
//    and responses an external interrupt on GPIO13 from ADS833x to recevie the conversion data.
//    This example can configure the internal CMR and CFR registers of ADS833x by SPI interface too.
//       
//    This example program is developed and demonstrated with Code Composer Studio V3.3 based on the 
//    eZdspTM F28335 from Spectrum Digital Incorporated and the ADS8332EVM from Texas Instruments.
//============================================================================= 
// Hardware connection between TMS320F28335 and ADS8332:
// F28335  GPIO16(SPI-A slave in, master out)----> ADS833x SDI 
// F28335  GPIO17(SPI-A master in, slave out)----> ADS833x SDO 
// F28335  GPIO18(SPI-A clock)----> ADS833x SCLK 
// F28335  GPIO19(/SPISTEA)----> ADS833x /CS 
// F28335  GPIO13 ----> ADS833x /INT 
// F28335  GPIO12 ----> ADS833x /Convst 
// F28335  GPIO7  ----> ADS833x /Reset 
//=============================================================================
// F28335 SPI setup in spi_init():    
// SPICCR(CLOCK POLARITY)   SPICTL(CLOCK PHASE)       
// 0x000F (0)               0x0006 (0)   (Data output:rising edge; Data input:falling edge of the SPICLK)   
// 0x004F (1)               0x000E (1)   (Data output:half-cycle ahead of the falling edge; Data input:
//                                        falling edge of the SPICLK)    
//=============================================================================
// Watch Variables:
// ADS833x_Result:  ADS8332 conversion data
// ADS833x_Buf[1]:  ADS8332 CFR register content  
//#############################################################################

#include "DSP2833x_Device.h"     
#include "DSP2833x_Examples.h"   

#define Convst_Low  1      // 0.040 = Minimum 40ns for ADS8332 /CONVST Low Pulse
#define BUF_SIZE    5      // Sample buffer size
#define SIZE   8192        // ADC Result buffer size
#define Sampling_Period   1000    // 1000us for 1ksps sampling speed

#pragma DATA_SECTION(ADS833x_Result,"DMARAML6");
#pragma DATA_SECTION(ADS833x_Buf,"DMARAML4");

interrupt void xint1_isr(void);
interrupt void cpu_timer0_isr(void);

Uint16 ADS833x_Buf[BUF_SIZE]; 
Uint16 ADS833x_Result[SIZE];  
// ADS8332 CFR: internal OSC/Manual-Trigger/250ksps/INT active low/auto nap disabled/nap&deep nap disabled/TAG disabled
Uint16 ADS833x_CFR_Reg = 0xFBD;      // 0xFBD for Auto channel select and Manual-Trigger,/INT function
                                     // 0x7BD for Manual channel select and Manual-Trigger,/INT function
Uint16 ADS833x_Write_Com = 0xE000;   // Command for writing CFR registers
Uint16 Reg_Word;   
Uint16 LoopCount;  
Uint16 Channel_Number;   

volatile Uint32 Xint1Count;  
volatile Uint32 TimerCount; 
volatile Uint16 Flag;       

void delay_loop(void);
void Gpio_setup(void);
void spi_init(void);
void ADS833x_init(void);
void ADS833x_Reset(void);
void ADS833x_Channel_Select(Uint16 Data);
void ADS833x_WriteRegister(Uint16 Data);
Uint16 ADS833x_ReadRegister();
Uint16 ADS833x_ReadData();
void error(void);

void main(void)
{
	Uint16 n;

// Step 1. Initialize System Control:
// PLL, WatchDog, enable Peripheral Clocks
// This example function is found in the DSP2833x_SysCtrl.c file.
   InitSysCtrl();
   
// Step 2. Initalize GPIO: 
// This example function is found in the DSP2833x_Gpio.c file and
// illustrates how to set the GPIO to it's default state.

// Step 3. Clear all interrupts and initialize PIE vector table:
   DINT;  // Disable CPU interrupts 

// Initialize PIE control registers to their default state.
// The default state is all PIE interrupts disabled and flags are cleared.  
// This function is found in the DSP2833x_PieCtrl.c file.
   InitPieCtrl();

   IER = 0x0000;
   IFR = 0x0000;

// Initialize the PIE vector table with pointers to the shell Interrupt Service Routines (ISR).  
// This will populate the entire table, even if the interrupt is not used in this example.
// The shell ISR routines are found in DSP2833x_DefaultIsr.c.
// This function is found in DSP2833x_PieVect.c.
   InitPieVectTable();

// Interrupts that are used in this example are re-mapped to
// ISR functions found within this file.  
   EALLOW;	// This is needed to write to EALLOW protected registers
   PieVectTable.XINT1 = &xint1_isr;
   PieVectTable.TINT0 = &cpu_timer0_isr;
   EDIS;   // This is needed to disable write to EALLOW protected registers
	
// Step 4. Initialize all the Device Peripherals:
// This function is found in DSP2833x_InitPeripherals.c
// This function can be found in DSP2833x_CpuTimers.c
    InitCpuTimers();   // For this example, only initialize the Cpu Timers

#if (CPU_FRQ_150MHZ)
// Configure CPU-Timer 0 to interrupt every Sampling_Period:
// 150MHz CPU Freq, 50 millisecond Period (in uSeconds)
   ConfigCpuTimer(&CpuTimer0, 150, Sampling_Period);
#endif
#if (CPU_FRQ_100MHZ)
// Configure CPU-Timer 0 to interrupt every Sampling_Period:
// 100MHz CPU Freq, 50 millisecond Period (in uSeconds)
   ConfigCpuTimer(&CpuTimer0, 100, Sampling_Period);
#endif

// To ensure precise timing, use write-only instructions to write to the entire register. Therefore, if any
// of the configuration bits are changed in ConfigCpuTimer and InitCpuTimers (in DSP2833x_CpuTimers.h), the
// below settings must also be updated.
   CpuTimer0Regs.TCR.all = 0x4001;  

// Step 5. User specific code, enable interrupts:
    Xint1Count = 0;   // Count XINT1 interrupts
	TimerCount = 0;   // Count Timer0 interrupts
    LoopCount = 0;    // Count times through idle loop
	Flag = 0;       
	Reg_Word = 0;   
	 
	// Initialize Buffer Tables
   for (n=0; n<BUF_SIZE; n++)
   {
    ADS833x_Buf[n] = n;
   }

   for (n=0; n<SIZE; n++)
   {
    ADS833x_Result[n] = n;
   } 
     
	Gpio_setup();
	InitSpiGpio();  
    spi_init();   	 
	ADS833x_Reset();
	ADS833x_init();

	// Enable XINT1 and TINT0 in the PIE: Group 1 interrupt 4 & 7
	// Enable INT1 which is connected to WAKEINT:
    PieCtrlRegs.PIECTRL.bit.ENPIE = 1;          // Enable the PIE block
    PieCtrlRegs.PIEIER1.bit.INTx4 = 1;          // Enable PIE Gropu 1 INT4
	// Enable TINT0 in the PIE: Group 1 interrupt 7
    PieCtrlRegs.PIEIER1.bit.INTx7 = 1;

    IER |= M_INT1;     // Enable CPU INT1
	// Enable global Interrupts and higher priority real-time debug events:
    EINT;   // Enable Global interrupt INTM
    ERTM;   // Enable Global realtime interrupt DBGM                                    

    // Set input qualifcation period for GPIO13 (/INT pin on ADS833x)
    EALLOW;
    GpioCtrlRegs.GPAMUX1.bit.GPIO13 = 0;  
    GpioCtrlRegs.GPADIR.bit.GPIO13 = 0;      // GPIO13 = input
    GpioCtrlRegs.GPAQSEL1.bit.GPIO13=0;      // Xint1 Synch to SYSCLKOUT only
    GpioCtrlRegs.GPACTRL.bit.QUALPRD1=0xFF;  
    EDIS;

    // Make GPIO13 the input source for XINT1
    EALLOW;
    GpioIntRegs.GPIOXINT1SEL.bit.GPIOSEL = 13;   // XINT1 connected to GPIO13
    EDIS;
    // Configure XINT1
    XIntruptRegs.XINT1CR.bit.POLARITY = 0;      // Falling edge interrupt
    // Enable XINT1 
    XIntruptRegs.XINT1CR.bit.ENABLE = 1;        // Enable XINT1

// Step 6. IDLE loop:	
    // Select input Channel if Manual Channel Selection is enabled in CFR register
	ADS833x_Buf[2] = ADS833x_CFR_Reg & 0x0800; 
	if (ADS833x_Buf[2] == 0)
	{
    Channel_Number = 0;      // Should be 0~7,Channel number for Manual channel select   
    ADS833x_Channel_Select(Channel_Number);   // Select ADC input channel
    }

    StartCpuTimer0();
     
   for(;;)
   {
	for(LoopCount=0;LoopCount<SIZE;LoopCount++)   
    {

    while(Flag == 0) { };   // Waiting for interrupt service finished
    Flag = 0;
	}

	LoopCount = 0; 
	Xint1Count = 0;
	TimerCount = 0; 
   }  
} 	


// Step 7. Insert all local Interrupt Service Routines (ISRs) and functions here:	

interrupt void xint1_isr(void)
{
    ADS833x_Result[LoopCount] = ADS833x_ReadData();	 	

	Flag = 1;
	Xint1Count++;
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;
}

interrupt void cpu_timer0_isr(void)
{
    CpuTimer0.InterruptCount++;

    GpioDataRegs.GPACLEAR.bit.GPIO12 = 1;     
   	DELAY_US(Convst_Low);   	   
    GpioDataRegs.GPASET.bit.GPIO12 = 1;       

	TimerCount++;
    PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;
}

void Gpio_setup(void)
{ 
    EALLOW; 
    // Enable an GPIO output on GPIO12, set it high -ADS833x /Convst
    GpioCtrlRegs.GPAPUD.bit.GPIO12 = 0;     
    GpioDataRegs.GPASET.bit.GPIO12 = 1;     
    GpioCtrlRegs.GPAMUX1.bit.GPIO12 = 0;    
    GpioCtrlRegs.GPADIR.bit.GPIO12 = 1;     // GPIO12 = output

    // Enable an GPIO output on GPIO7, set it high -ADS833x /Reset
    GpioCtrlRegs.GPAPUD.bit.GPIO7 = 0;     
    GpioDataRegs.GPASET.bit.GPIO7 = 1;     
    GpioCtrlRegs.GPAMUX1.bit.GPIO7 = 0;    
    GpioCtrlRegs.GPADIR.bit.GPIO7 = 1;     // GPIO7 = output 
    EDIS;
}

void spi_init()
{    
    SpiaRegs.SPICCR.all =0x004F;  // Reset on, falling edge, 16-bit char bits
  	SpiaRegs.SPICTL.all =0x000E;  // Enable master mode,SPICLK signal delayed by one half-cycle,Enable RECEIVER OVERRUN interrupt,enable talk and SPI int disabled
                                                 
	SpiaRegs.SPIBRR =0x000A;					 

	SpiaRegs.SPICCR.all = SpiaRegs.SPICCR.all|0x0080;  // Relinquish SPI from Reset by setting bit 7 to 1(SPI SW RESET) 
    SpiaRegs.SPIPRI.bit.FREE = 1;                
}


void ADS833x_Reset()
{
	// Reset ADS833x manually,must be held low for a minimum of 25ns to reset the device and return the device to default mode
    GpioDataRegs.GPACLEAR.bit.GPIO7 = 1;  	         
    DELAY_US(1);                              
    GpioDataRegs.GPASET.bit.GPIO7 = 1;    	
    //After reset,the minimum time before the device can be reconfigured by /CS going low and data clocking in on SDI is 2us
	DELAY_US(2);  
}

void ADS833x_init()      // Initialize ADS833x
{
    Reg_Word = ADS833x_Write_Com + ADS833x_CFR_Reg;
	ADS833x_WriteRegister(Reg_Word);    // ADS833x internal CFR register configuration

    while(SpiaRegs.SPISTS.bit.INT_FLAG !=1){}  // Wait until RX data is ready
                                                 
	ADS833x_Buf[0] = ADS833x_ReadRegister();
    ADS833x_Buf[1] = ADS833x_Buf[0] & 0x0FFF;
    if(ADS833x_CFR_Reg != ADS833x_Buf[1]) error( );  // Check that CFR register content
}

void ADS833x_WriteRegister(Uint16 Data)    // Write CMR and CFR register  
{
   	while(SpiaRegs.SPISTS.bit.BUFFULL_FLAG !=0) { }  // Wait until SPITXBUF is free 
    SpiaRegs.SPITXBUF = Data;                       
}

Uint16 ADS833x_ReadRegister()     // Read CFR register
{
	Uint16 rData0;    
	while(SpiaRegs.SPISTS.bit.BUFFULL_FLAG !=0) { }  // Wait until SPITXBUF is free 
    SpiaRegs.SPITXBUF= 0xC000;    

	while(SpiaRegs.SPISTS.bit.BUFFULL_FLAG !=0){}  // Wait until SPITXBUF is free 
	rData0 = SpiaRegs.SPIRXBUF;                    // Read RXBUF to clear INT_FLAG
    while(SpiaRegs.SPISTS.bit.INT_FLAG !=1){}      // Wait until RX data is ready
                                                 
	rData0 = SpiaRegs.SPIRXBUF;   
	return rData0;
}

Uint16 ADS833x_ReadData()      // Read conversion data 
{
	Uint16 rData;	
	while(SpiaRegs.SPISTS.bit.BUFFULL_FLAG !=0) { }  // Wait until SPITXBUF is free 
    SpiaRegs.SPITXBUF= 0xD000;    

 	while(SpiaRegs.SPISTS.bit.BUFFULL_FLAG !=0){}  // Wait until SPITXBUF is free 
	rData = SpiaRegs.SPIRXBUF;                     // Read RXBUF to clear INT_FLAG
    while(SpiaRegs.SPISTS.bit.INT_FLAG !=1){}      // Wait until RX data is ready

	rData = SpiaRegs.SPIRXBUF;   
  	return rData;    
}

    // Write CMR register to select channel if Manual channel select is enabled  
void ADS833x_Channel_Select(Uint16 Data)                
{
    Data = Data << 12;   
	while(SpiaRegs.SPISTS.bit.BUFFULL_FLAG !=0) { }  // Wait until SPITXBUF is free 
    SpiaRegs.SPITXBUF = Data;                       
}

void error(void)
{
    asm("ESTOP0");  // test failed!! Stop!
    for (;;);
}

//===========================================================================
// No more.
//===========================================================================
